/* ----------------------------------------------------------------
 *
 * tele.c
 *
 * CASCA - telsecope query routines
 *
 * ---------------------------------------------------------------- */

#ifndef DEBUG
#define DEBUG  1
#endif

/* ---------------------------------------------------------------- */

#define _REENTRANT

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <assert.h>

#include "casca.h"
#include "tcpip.h"
#include "utils.h"

#define PREFUN  __func__ 

/* ---------------------------------------------------------------- */

static int baade_sock=-1,clay_sock=-1,dimm_sock=-1,dupont_sock=-1;

/* ---------------------------------------------------------------- */

#if 0 // IDEA
enum telescope_type_enum { TELE_NONE,TELE_MAG,TELE_DIMM,TELE_DUPONT };

typedef struct telio_tag {
  int  sock,type;
  char host[128];
  double az,el;
} TeleStruct;

TeleStruct* tele_create(const char* host,int port,int type)
{
  TeleStruct *t = (TeleStruct*)malloc(sizeof(TeleStruct));
  strpcy(t->host,host);
  t->port = port;
  t->type = type;

  return t;
}
#endif

/* ---------------------------------------------------------------- */

static int get_param(int sock,const char* cmd,double* par)
{
  char com[256],buf[128];
#if (DEBUG > 1)
  fprintf(stderr,"%s(%d,%s,%p)\n",PREFUN,sock,cmd,par);
#endif

  assert(1+strlen(cmd) <= sizeof(com));
  sprintf(com,"%s\n",cmd);
  int err = TCPIP_Request3(sock,com,buf,sizeof(buf),2); 
  if (!err) *par = atof(buf);
  
  return err;
}

/* --- */

static int tcs_open(const char* host,int port)
{
  int err;

  int sock = TCPIP_CreateClientSocket(host,port,&err);
  if (err) { char buf[256];
    sprintf(buf,"opening connection to '%s:%d' failed, err=%d",host,port,err);
    logit(0,PREFUN,buf,2);
    assert(sock < 0);
  }
  return sock;
}

/* --- */

static int tcs_update(int sock,double* az,double* el,int env)
{
  int  err=0;
#if (DEBUG > 1)
  fprintf(stderr,"%s(%d,%p,%p,%d)\n",PREFUN,sock,az,el,env);
#endif
  assert(sock > 0);

  if (!err) err = get_param(sock,"telaz",az);
  if (!err) err = get_param(sock,"telel",el);
  if (env) {
    if (!err) err = get_param(sock,"wxtemp", &env_temp);
    if (!err) err = get_param(sock,"wxpres", &env_pres);
    if (!err) err = get_param(sock,"wxhumid",&env_humi);
    if (!err) err = get_param(sock,"wxwind", &env_wind);
    if (!err) err = get_param(sock,"wxwdir", &env_wdir);
  }
  if (err) { char host[64]="",buf[128];
    if      (sock == baade_sock)  strcpy(host,baade_host);
    else if (sock == clay_sock)   strcpy(host,clay_host);
    else if (sock == dupont_sock) strcpy(host,dupont_host);
    sprintf(buf,"requesting status from '%s' failed, err=%d",host,err);
    logit(0,PREFUN,buf,2);
  }
  return err;
}

/* --- */

double tele_query(void)
{
  int  err=0;
  char buf[256];

  double t1 = walltime(0);

  sprintf(buf,"%s,%s,%s,%s",baade_host,clay_host,dimm_host,dupont_host);
  logit(0,PREFUN,buf,1);

  env_temp = env_pres = env_humi = env_wind = env_wdir = 0;
  baade_az = baade_el = 0;
  if (*baade_host) {
    if (baade_sock < 0) {
      baade_sock = tcs_open(baade_host,5800);
    }
  }
  if (baade_sock > 0) {
    err = tcs_update(baade_sock,&baade_az,&baade_el,1);
    if (err) { close(baade_sock); baade_sock = -1; }
  }      

  clay_az = clay_el = 0;
  if (*clay_host) {
    if (clay_sock < 0) {
      clay_sock = tcs_open(clay_host,5800);
    }
  }
  if (clay_sock > 0) {
    err = tcs_update(clay_sock,&clay_az,&clay_el,(env_pres==0) ? 1: 0);
    if (err) { close(clay_sock); clay_sock = -1; }
  }

  dupont_az = dupont_el = 0;
  if (*dupont_host) {
    if (dupont_sock < 0) {
      dupont_sock = tcs_open(dupont_host,4242);
    }
  }
  if (dupont_sock > 0) {
    err = tcs_update(dupont_sock,&dupont_az,&dupont_el,0);
    if (err) { close(dupont_sock); dupont_sock = -1; }
  }

  dimm_az = dimm_el = 0;
  if (*dimm_host) {
    if (dimm_sock < 0) {
      dimm_sock = tcs_open(dimm_host,51001);
    }
  }
  if (dimm_sock > 0) {
    err = TCPIP_Request3(dimm_sock,"report state",buf,sizeof(buf),2);
    if (!err) { int i; char cmd[128];
      int n = atoi(buf);
      for (i=0; i<n; i++) { 
        sprintf(cmd,"report state %d",1+i);
        err = TCPIP_Request3(dimm_sock,cmd,buf,sizeof(buf),2);
        if (err) break;
        if (!strncmp(buf,"alt=",4)) {
          sscanf(buf,"alt= %lf, az= %lf",&dimm_el,&dimm_az);
          break;
        }
      } 
    }
    if (err) {
      sprintf(buf,"requesting status from 'DIMM' failed, err=%d",err);
      logit(0,PREFUN,buf,2);
      close(dimm_sock); dimm_sock = -1;
    }
  }

  double t2 = walltime(0);

  return t2-t1;
}

/* ---------------------------------------------------------------- */

void tele_close(void)
{
#if (DEBUG > 0)
  fprintf(stderr,"%s: %d,%d,%d,%d\n",PREFUN,
          baade_sock,clay_sock,dimm_sock,dupont_sock);
#endif
  if (baade_sock  > 0) { close(baade_sock);  baade_sock  = -1; }
  if (clay_sock   > 0) { close(clay_sock);   clay_sock   = -1; }
  if (dimm_sock   > 0) { close(dimm_sock);   dimm_sock   = -1; }
  if (dupont_sock > 0) { close(dupont_sock); dupont_sock = -1; }
}

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */

